/*
 *	rs2analyzer.c
 *		Copyright (C)2002-2004 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "errmes.h"
#include "utility.h"
#include "filesfc.h"
#include "rs2monster.h"
#include "rs2character.h"
#include "rs2item.h"
#include "rs2tech.h"
#include "rs2form.h"

#define FILENAME_MON_PARAM			"rs2monparam.txt"
#define FILENAME_MON_REGIST			"rs2monregist.txt"
#define FILENAME_MON_FLAG			"rs2monflag.txt"
#define FILENAME_MON_ITEM			"rs2monitem.txt"
#define FILENAME_MON_EQUIP			"rs2monequip.txt"
#define FILENAME_MON_ACT			"rs2monact.txt"
#define FILENAME_CHR_PARAM			"rs2chrparam.txt"
#define FILENAME_CHR_FLAG			"rs2chrflag.txt"
#define FILENAME_CHR_SP_PARAM		"rs2chrspparam.txt"
#define FILENAME_CHR_SP_FLAG		"rs2chrspflag.txt"
#define FILENAME_CHR_SP_EQUIP		"rs2chrspequip.txt"
#define FILENAME_CHR_LVUP			"rs2chrlvup.txt"
#define FILENAME_ITEM_PARAM			"rs2itemparam.txt"
#define FILENAME_ITEM_REGIST		"rs2itemregist.txt"
#define FILENAME_ITEM_EFFECT		"rs2itemeffect.txt"
#define FILENAME_TECH_PARAM			"rs2techparam.txt"
#define FILENAME_TECH_LEARN_FLAG	"rs2techlearnflag.txt"
#define FILENAME_TECH_LEARN_DATA	"rs2techlearndata.txt"
#define FILENAME_TECH_EVADE_DATA	"rs2techevadedata.txt"
#define FILENAME_FORM_DATA			"rs2formdata.txt"

#define STR_TOO_FEW_ARGUMENTS		"Too few arguments"
#define STR_INVALID_OPTION			"Invalid option"
#define STR_INVALID_ROMIMAGE_FILE	"Invalid ROM-Image file"
#define STR_CANNOT_CREATE_FILE		"Cannot create file"

#define STR_TITLE	"ROMANCING SAGA2      "
			/* 0x52, 0x4f, 0x4d, 0x41, 0x4e, 0x43, 0x49, 0x4e, 0x47, 0x20, 0x53, 0x41, 0x47, 0x41, 0x32, 0x20, */
			/* 0x20, 0x20, 0x20, 0x20, 0x20 */

/* 쌠\L */
static const char *str_copyright = "rs2analyzer - Copyright (C)2002-2004 Shingo Endo <s-endo@yk.rim.or.jp>";

int main(int argc, char **argv)
{
	int i;
	int ret = EXIT_SUCCESS;
	int flag_invalid = 0;
	int flag_usage   = 0;
	char str_srcfilename[1024] = "";
	char str_dstpath[1024]     = "";
	char str_dstfilename[1024] = "";
	mondata_t   *buf_mondata   = NULL;
	chrdata_t   *buf_chrdata   = NULL;
	chrdatasp_t *buf_chrdatasp = NULL;
	chrlvup_t   *buf_chrlvup   = NULL;
	itemdata_t  *buf_itemdata  = NULL;
	techdata_t  *buf_techdata  = NULL;
	techlearn_t *buf_techlearn = NULL;
	formdata_t  *buf_formdata  = NULL;

	errno = 0;
	err_setprogname(argv[0]);

	/* R}hC͂ */
	for (i = 1; i < argc; i++)
	{
		if (argv[i] == NULL)
			continue;

		if (*argv[i] == '-')
		{
			const char *ptr = argv[i];

			while (*++ptr != '\0')
			{
				switch (tolower(*ptr))
				{
				case 'h':	/* gp@̕\ */
					flag_usage = 1;
					break;
				default:
					flag_invalid = 1;
					break;
				}
			}
		}
		else
		{
			if (*str_srcfilename == '\0')
			{
				strncpy(str_srcfilename, argv[i], sizeof(str_srcfilename));
				str_srcfilename[sizeof(str_srcfilename) - 1] = '\0';
			}
			else if (*str_dstpath == '\0')
				transpath(str_dstpath, argv[i], sizeof(str_dstpath));
		}
	}

	/* gp@\ */
	if (flag_usage)
	{
		puts(str_copyright);
		printf("%s: [option] src-pathname [dst-pathname]\n", err_getprogname());
		printf("\t-h\tdisplay this help and exit\n");
		printf("\n");

		exit(EXIT_SUCCESS);
	}

	/* R}hC`FbN */
	if (flag_invalid)
		err_printf(STR_INVALID_OPTION);
	if (*str_srcfilename == '\0')
		err_printf(STR_TOO_FEW_ARGUMENTS);

	/* t@CI[v */
	if (!file_open_sfc(str_srcfilename, STR_TITLE))
		err_printf(STR_INVALID_ROMIMAGE_FILE " - %s", str_srcfilename);

	/* ef[^́E͂ */
	buf_mondata   = mon_readdata();
	buf_chrdata   = chr_readdata();
	buf_chrdatasp = chr_readdatasp();
	buf_chrlvup   = chr_readlvup();
	buf_itemdata  = item_readdata();
	buf_techdata  = tech_readdata();
	buf_techlearn = tech_readlearn();
	buf_formdata  = form_readdata();

	/* ef[^o͂ */
	if (buf_mondata   == NULL ||
		buf_chrdata   == NULL ||
		buf_chrdatasp == NULL ||
		buf_chrlvup   == NULL ||
		buf_itemdata  == NULL ||
		buf_techdata  == NULL ||
		buf_techlearn == NULL ||
		buf_formdata  == NULL)
	{
		wrn_printf(STR_INVALID_ROMIMAGE_FILE " - %s*", str_srcfilename);
		ret = EXIT_FAILURE;
	}
	else
	{
		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_MON_PARAM);
		if (!mon_writeparam(str_dstfilename, buf_mondata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_MON_REGIST);
		if (!mon_writeregist(str_dstfilename, buf_mondata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_MON_FLAG);
		if (!mon_writeflag(str_dstfilename, buf_mondata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_MON_ITEM);
		if (!mon_writeitem(str_dstfilename, buf_mondata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_MON_EQUIP);
		if (!mon_writeequip(str_dstfilename, buf_mondata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_MON_ACT);
		if (!mon_writeact(str_dstfilename, buf_mondata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_CHR_PARAM);
		if (!chr_writeparam(str_dstfilename, buf_chrdata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_CHR_FLAG);
		if (!chr_writeflag(str_dstfilename, buf_chrdata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_CHR_SP_PARAM);
		if (!chr_writespparam(str_dstfilename, buf_chrdatasp))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_CHR_SP_FLAG);
		if (!chr_writespflag(str_dstfilename, buf_chrdatasp))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}
#if 0
		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_CHR_SP_EQUIP);
		if (!chr_writespequip(str_dstfilename, buf_chrdatasp))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}
#endif
		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_CHR_LVUP);
		if (!chr_writelvup(str_dstfilename, buf_chrlvup))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_ITEM_PARAM);
		if (!item_writeparam(str_dstfilename, buf_itemdata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_ITEM_REGIST);
		if (!item_writeregist(str_dstfilename, buf_itemdata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_ITEM_EFFECT);
		if (!item_writeeffect(str_dstfilename, buf_itemdata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}
#if 0
		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_TECH_PARAM);
		if (!tech_writeparam(str_dstfilename, buf_techdata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}
#endif
		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_TECH_LEARN_FLAG);
		if (!tech_writelearnflag(str_dstfilename, buf_techdata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_TECH_LEARN_DATA);
		if (!tech_writelearndata(str_dstfilename, buf_techlearn))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_TECH_EVADE_DATA);
		if (!tech_writeevadedata(str_dstfilename, buf_techdata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}

		sprintf(str_dstfilename, "%s%s", str_dstpath, FILENAME_FORM_DATA);
		if (!form_writedata(str_dstfilename, buf_formdata))
		{
			wrn_printf(STR_CANNOT_CREATE_FILE " - %s:", str_dstfilename);
			ret = EXIT_FAILURE;
		}
	}

	/* t@CN[Y */
	file_close_sfc();

	/* obt@ */
	tech_freelearn(buf_techlearn);
	free(buf_mondata);
	free(buf_chrdata);
	free(buf_chrdatasp);
	free(buf_itemdata);
	free(buf_techdata);
	free(buf_techlearn);
	free(buf_formdata);
	buf_mondata   = NULL;
	buf_chrdata   = NULL;
	buf_chrdatasp = NULL;
	buf_itemdata  = NULL;
	buf_techdata  = NULL;
	buf_techlearn = NULL;
	buf_formdata  = NULL;

	return ret;
}
