/*
 *	rs2form.c
 *		Copyright (C)2002, 2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "rs2decode.h"
#include "rs2define.h"
#include "rs2form.h"

#define FILEPOS_FORMDATA		0x1e7512
#define FILEPOS_FORMNAME		0x1de260
#define NUM_FORMATION			16
#define SIZEOF_FORMDATA			16
#define SIZEOF_FORMNAME			10

static void form_setname(void);

static char tbl_formname[NUM_FORMATION][256];	/* w`̖Oi[e[u */
static int flag_setformname = 0;				/* tbl_formnameɃf[^i[Ă邩ǂ */

const char *form_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_FORMATION <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setformname)
		form_setname();

	return tbl_formname[index];
}

static void form_setname(void)
{
	int i;

	/* w`̖Of[^ */
	for (i = 0; i < NUM_FORMATION; i++)
		*tbl_formname[i] = '\0';

	/* w`̖Of[^ǂݍ */
	for (i = 0; i < NUM_FORMATION; i++)
	{
		unsigned char buf[SIZEOF_FORMNAME];

		file_read_sfc(buf, FILEPOS_FORMNAME + SIZEOF_FORMNAME * i, SIZEOF_FORMNAME);
		dec_string(tbl_formname[i], buf, sizeof(tbl_formname[i]), SIZEOF_FORMNAME);
	}

	flag_setformname = 1;
}

formdata_t *form_readdata(void)
{
	int i, j;
	formdata_t *buf_formdata = NULL;

	/* w`̊ef[^i[obt@mۂ */
	if ((buf_formdata = malloc(sizeof(formdata_t) * NUM_FORMATION)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* w`̊ef[^ǂݍ */
	for (i = 0; i < NUM_FORMATION; i++)
	{
		unsigned char buf[SIZEOF_FORMDATA];

		file_read_sfc(buf, FILEPOS_FORMDATA + SIZEOF_FORMDATA * i, SIZEOF_FORMDATA);

		for (j = 0; j < 5; j++)
		{
			/* lnf[^ */
			buf_formdata[i].pos[j].targeted_rate  = buf[j * 3 + 1] - buf[j * 3] + 1;
			buf_formdata[i].pos[j].increase_speed = (((buf[j * 3 + 2] & 0x04) != 0x00) ? -1 : 1) * (buf[j * 3 + 2] & 0x03);

			/* tOnf[^ */
			buf_formdata[i].pos[j].flag_defense_before        = ((buf[j * 3 + 2] & 0x80) != 0x00);
			buf_formdata[i].pos[j].flag_defense_after         = ((buf[j * 3 + 2] & 0x40) != 0x00);
			buf_formdata[i].pos[j].flag_attack_short_distance = ((buf[j * 3 + 2] & 0x20) != 0x00);
			buf_formdata[i].pos[j].flag_attack_long_distance  = ((buf[j * 3 + 2] & 0x10) != 0x00);
			buf_formdata[i].pos[j].flag_attack_magic          = ((buf[j * 3 + 2] & 0x08) != 0x00);
		}
	}

	return buf_formdata;
}

int form_writedata(const char *ptr_filename, const formdata_t *ptr_formdata)
{
	int i, j;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_formdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* w`̐lnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* w`̐lnf[^o͂ */
	for (i = 0; i < NUM_FORMATION; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%03d(%02X): %s]\n", i, i, form_getname(i));

		fprintf(fp, "No. I f "
					" Oh h ߍU U pU"
					" h()\n");
		for (j = 0; j < 5; j++)
		{
			fprintf(fp, "%02d:  %5d, %+5d,  %2s   %2s   %2s   %2s   %2s     %2s\n", j + 1,
						ptr_formdata[i].pos[j].targeted_rate,
						ptr_formdata[i].pos[j].increase_speed,
						str_getflag(ptr_formdata[i].pos[j].flag_defense_before),
						str_getflag(ptr_formdata[i].pos[j].flag_defense_after),
						str_getflag(ptr_formdata[i].pos[j].flag_attack_short_distance),
						str_getflag(ptr_formdata[i].pos[j].flag_attack_long_distance),
						str_getflag(ptr_formdata[i].pos[j].flag_attack_magic),
						str_getflag(ptr_formdata[i].pos[j].increase_speed % 2 != 0));
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
